;  ------------------------------------------------------------------

Proc Wait_Proc_Many_Win(t, e, c)

;  Wait procedure which handles a multi-window desktop,
;  and passes through most events to Paradox

;  Parameters:
;     t = trigger type
;     e = event record
;     c = cycle number

Private
   k,           ;key code
   mt           ;menu tag

If is_event_trace then
   Wait_Trace(t,e,c)
Endif

Switch
   Case t="EVENT" and e["Type"]="MESSAGE" and e["Message"]="MENUSELECT":
      mt = e["MenuTag"]
      If mt="Quit" then Return 2 Endif
      Menu_Many_Win(mt)
   Case t="EVENT" and e["Type"]="KEY":
      k = e["KeyCode"]
      If k=asc("F2") or k=asc("CoEditKey") then
         ExecEvent e
         Return 1
      Endif
EndSwitch

Return 0

EndProc

;  ------------------------------------------------------------------

Proc Menu_Many_Win(mt)

;  Handle the menu selection during many-window data entry

;  Parameter:
;     mt = Menu Tag the user has selected

Private
   t2,          ;new table name
   tag_type,    ;what type of tag
   wh1,         ;current window handle
   wh2          ;new window handle


;  Is it window movement or table movement?

tag_type = substr(mt,1,1)    ;first letter of tag tells type of tag
mt = substr(mt,2,len(mt)-1)  ;the remaining letters tell the window/table


;  Window movement

Switch
   Case tag_type="W":           ;window movement

;  Determine the current and desired window handles

      wh1 = GetWindow()         ;current window handle
      wh2 = whs[mt]             ;desired window handle
      If wh1=wh2 then           ;we're already here
         Return                 ;nothing to do
      Endif

;  Try to move to the desired window

      Window Select wh2         ;select the window
      If GetWindow()=wh2 then   ;we succeeded
         Return                 ;nothing more to do
      Endif

      Beep Message "Can't get to that window with WINDOW SELECT"

;  Table movement

   Case tag_type="T":           ;table movement
   
;  Determine the desired table, and try to move to it

      t2 = ts[mt]               ;the desired table name
      If isformview() then      ;we're on a form
         FormKey                ;switch back to the table-view window
      Endif
      MoveTo t2                 ;move to the table
      
      If upper(table())=t2 then ;we succeeded
         Return                 ;nothing more to do
      Endif

      Beep Message "Can't get to that table with MOVETO"
EndSwitch            

EndProc

;  ------------------------------------------------------------------

Proc Open_Many_Windows()

;  Open a variety of different windows

Private
   wb           ;window attributes dynarray

Echo Normal     ;this might be fun to watch

DynArray whs[]                          ;a place to store window handles
DynArray ts[]                           ;a place to store table names
   
DynArray wb[]
wb["HasShadow"] = False
wb["Floating"]  = False

View "Student"
Window Handle Image imageno() to wh
whs["Stu"] = wh
ts["Stu"]  = upper(table())
Window Select wh
Window Move wh to 1000,1000             ;push it off the screen
PickForm 1
wh = GetWindow()
whs["StuF"] = wh
Window SetAttributes wh from wb
Window Resize wh to 11,34
Window Move wh to 1,1

MoveTo [Comments]                       ;memo field
FieldView                               ;open the memo field window
wh = GetWindow()
whs["Memo"] = wh
Window SetAttributes wh from wb
Window Resize wh to 5,41
Window Move wh to 1,37

FieldView                               ;return to the form
CtrlHome                                ;return to page 1 of the form

View "Order"
wh = GetWindow()
whs["Ord"] = wh
ts["Ord"]  = upper(table())
Window Move wh to 1000,1000             ;push it off the screen
PickForm 1
wh = GetWindow()
whs["OrdF"] = wh
Window SetAttributes wh from wb
Window Resize wh to 10,44
Window Move wh to 13,0

View "Customer"
wh = GetWindow()
whs["Cust"] = wh
ts["Cust"]  = upper(table())
Window SetAttributes wh from wb
Window Resize wh to 6,34
Window Move wh to 17,46

View "Dept"
wh = GetWindow()
whs["Dept"] = wh
ts["Dept"]  = upper(table())
Window SetAttributes wh from wb
Window Resize wh to 6,34
Window Move wh to 10,46

Editor Open "X.Txt"
wh = GetWindow()
whs["Edit"] = wh
wb["Title"] = "X.Txt"
Window SetAttributes wh from wb
Window Resize wh to 5,23
Window Move wh to 7,36

wb["Title"] = "A Canvas"
Window Create Attributes wb
   @ 7,61 Height 5 Width 18 to wh
   @ 0,0 ?? "This is a" ? "Canvas"
whs["Canv"] = wh

Echo Off

ShowPullDown
   "Window":  "Select a Window":        "Window"
      SubMenu
         "Student Form":  "The 2-Page Student Multi-Table Form":  "WStuF",
         "Order Form":    "The Order Multi-Table Form":           "WOrdF",
         "Customer":      "The Customer Table":                   "WCust",
         "Department":    "The Dept Table":                       "WDept",
         "Comments":      "The Student->Comments Memo Field":     "WMemo",
         "Editor":        "The X.TXT File in the Editor Window":  "WEdit",
         "Canvas":        "The Canvas-Only Window":               "WCanv",
         "Student Table": "The Student Table View":               "WStu",
         "Order Table":   "The Order Table View":                 "WOrd",
         "Event Trace":   "The Event-Trace Window":               "WEvTr"
      EndSubMenu,
   "Table":   "Select a Table":         "Table"
      SubMenu
         "Customer":      "The Customer Table":                   "TCust",
         "Department":    "The Dept Table":                       "TDept",
         "Student Table": "The Student Table View":               "TStu",
         "Order Table":   "The Order Table View":                 "TOrd"
      EndSubMenu,
   "Quit":    "All Done":               "Quit"
EndMenu

If not is_event_trace then
   MenuDisable "EvTr"
Else
   whs["EvTr"] = evtrace_params["Window Handle"]
Endif

EndProc

;  ------------------------------------------------------------------

Proc Many_Window_Main()

;  The main procedure with the WAIT command

Private
   ts,          ;dynamic array with table names
   whs          ;dynamic array with window handles

;  Initialize the desktop

Open_Many_Windows()

;  The WAIT command

Wait Workspace
   Proc "Wait_Proc_Many_Win"
   Mouse "Down", "Up"
   Key "All"
   Message "All"
   Trigger "All"
EndWait

Quit "Wait retval="+strval(retval)

EndProc

;  ------------------------------------------------------------------

Many_Window_Main()

;  ------------------------------------------------------------------
